
class Point:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __add__(self, other):
        return Point(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return Point(self.x - other.x, self.y - other.y)

    def __div__(self, denom):
        return Point(self.x / denom, self.y / denom)

    def __mul__(self, denom):
        return Point(self.x * denom, self.y * denom)

class Rectangle:
    def __init__(self, top, right, bottom, left):
        self.top = top
        self.right = right
        self.bottom = bottom
        self.left = left

    @classmethod
    def containing(cls, *points):
        return cls(min([p.y for p in points]),
                   max([p.x for p in points]),
                   max([p.y for p in points]),
                   min([p.x for p in points]))

    def enlarged(self, w, h = None):
        if h is None:
            h = w
        return Rectangle(self.top - h, self.right + w, self.bottom + h, self.left - h)

    def __or__(self, other):
        return Rectangle(min(self.bottom, other.bottom, self.top, other.top),
                         max(self.right, other.right, self.left, other.left),
                         max(self.bottom, other.bottom, self.top, other.top),
                         min(self.right, other.right, self.left, other.left))

    def __ior__(self, other):
        self.top = min(self.bottom, other.bottom, self.top, other.top)
        self.right = max(self.right, other.right, self.left, other.left)
        self.bottom = max(self.bottom, other.bottom, self.top, other.top)
        self.left = min(self.right, other.right, self.left, other.left)
        return self

    @property
    def tc(self):
        return Point((self.left + self.right) / 2, self.top)

    @property
    def bc(self):
        return Point((self.left + self.right) / 2, self.bottom)

    @property
    def lc(self):
        return Point(self.left, (self.top + self.bottom) / 2)

    @property
    def rc(self):
        return Point(self.right, (self.top + self.bottom) / 2)

    @property
    def center(self):
        return Point((self.left + self.right) / 2, (self.top + self.bottom) / 2)

    @property
    def tl(self):
        return Point(self.left, self.top)

    @property
    def tr(self):
        return Point(self.right, self.top)

    @property
    def bl(self):
        return Point(self.left, self.bottom)

    @property
    def br(self):
        return Point(self.right, self.bottom)
